package b0

  1. Overview
  2. Docs
On This Page
  1. Program execution
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module B0_unit.ActionSource

Program execution

exec_cwd is a function to determine a current working directory for exec actions.

scope_cwd is function that determines the unit's scope.

exec_env is a function to determine an environement for exec actions.

Sourceval exec : action

exec is an action that B0_std.Os.Exit.exec's a unit's outcome as follows:

exec_file u file argv is a B0_std.Os.Exit.exec with file and argv and: